@@ -1,24 +1,30 @@
-from os import startfile, getcwd
-from os.path import join
+from os import (startfile,
+                getcwd)
+from os.path import (join,
+                     isfile)
+from csv import (writer,
+                 excel)
+from openpyxl import (Workbook,
+                      load_workbook)
 from datetime import (datetime,
                       date)
 from statistics import (mean,
                         variance,
                         stdev)
-from plot import Plot
-from timber import (TimberQuick,
-                    TimberFull)
-from _constants import (math,
-                        extension_check,
-                        LOG_LENGTHS)
-from _import_from_sheets import (import_csv_quick,
-                                 import_csv_full,
-                                 import_excel_quick,
-                                 import_excel_full)
-from _console_print import (print_species,
-                            print_logs,
-                            print_species_stats)
-from _pdf_print import PDF
+from treetopper.plot import Plot
+from treetopper.timber import (TimberQuick,
+                               TimberFull)
+from treetopper._constants import (math,
+                                   extension_check,
+                                   LOG_LENGTHS)
+from treetopper._import_from_sheets import (import_csv_quick,
+                                            import_csv_full,
+                                            import_excel_quick,
+                                            import_excel_full)
+from treetopper._console_print import (print_species,
+                                       print_logs,
+                                       print_species_stats)
+from treetopper._pdf_print import PDF
 
 
 
@@ -67,7 +73,7 @@ class Stand(object):
 
        ** these files need to be formatted correctly **
 
-       To create a formatted blank CSV or Excel file, open the terminal and type "python treetopper.blank_sheet.py" and
+       To create a formatted blank CSV or Excel file, open the terminal and type "python -m treetopper.blank_sheet" and
        run through the prompts
 
        Once data has been added to the sheet, instantiate a Stand and call either:
@@ -130,6 +136,8 @@ class Stand(object):
 
         self.logs = {}
 
+        self.table_data = []
+
         self.metrics = ['tpa', 'ba_ac', 'rd_ac', 'bf_ac', 'cf_ac']
         self.attrs = ['_gross', '_stats', '']
 
@@ -180,6 +188,7 @@ class Stand(object):
         self.vbar = self.bf_ac / self.ba_ac
         self._update_species(plot)
         self._update_logs(plot)
+        self.table_data = self._update_table_data()
 
     def from_csv_quick(self, file_path):
         """Imports tree and plot data from a CSV file for a quick cruise and adds that data to the stand"""
@@ -245,6 +254,77 @@ class Stand(object):
                 plot.add_tree(tree)
             self.add_plot(plot)
 
+    def table_to_csv(self, filename: str, directory: str = None):
+        """Creates or appends a CSV file with tree data from self.table_data"""
+        check = extension_check(filename, '.csv')
+        if directory:
+            file = join(directory, check)
+        else:
+            file = join(getcwd(), check)
+
+        if isfile(file):
+            allow = 'a'
+            start = 1
+        else:
+            allow = 'w'
+            start = 0
+
+        with open(file, allow, newline='') as csv_file:
+            csv_write = writer(csv_file, dialect=excel)
+            for i in self.table_data[start:]:
+                csv_write.writerow(i)
+
+    def table_to_excel(self, filename: str, directory: str = None):
+        """Creates or appends an Excel file with tree data from self.table_data"""
+        check = extension_check(filename, '.xlsx')
+        if directory:
+            file = join(directory, check)
+        else:
+            file = join(getcwd(), check)
+
+        if isfile(file):
+            wb = load_workbook(file)
+            ws = wb.active
+            for i in self.table_data[1:]:
+                ws.append(i)
+            wb.save(file)
+        else:
+            wb = Workbook()
+            ws = wb.active
+            for i in self.table_data:
+                ws.append(i)
+            wb.save(file)
+
+    def _update_table_data(self):
+        heads = ['Stand', 'Plot Number', 'Tree Number', 'Species', 'DBH', 'Height',
+                 'Stump Height', 'Log 1 Length', 'Log 1 Grade', 'Log 1 Defect', 'Between Logs Feet']
+        master = []
+        max_logs = []
+        for i, plot in enumerate(self.plots):
+            for j, tree in enumerate(plot.trees):
+                temp = [self.name, i + 1, j + 1]
+                for key in ['species', 'dbh', 'height']:
+                    temp.append(tree[key])
+                len_logs = len(tree.logs)
+                max_logs.append(len_logs)
+                for k, lnum in enumerate(tree.logs):
+                    log = tree.logs[lnum]
+                    if lnum == 1:
+                        temp.append(log.stem_height - log.length - 1)
+                    for lkey in ['length', 'grade', 'defect']:
+                        temp.append(log[lkey])
+                    if k < len(tree.logs) - 1:
+                        between = tree.logs[lnum+1].stem_height - log.stem_height - tree.logs[lnum+1].length - 1
+                        if between < 0:
+                            temp.append(0)
+                        else:
+                            temp.append(between)
+                master.append(temp)
+        heads += self._add_logs_to_table_heads(max(max_logs))
+        master.insert(0, heads)
+        return master
+
+
     def _update_metrics(self, metric: str):
         """Updates stand metrics based on the metric entered in the argument, used internally"""
         metric_list = [plot[metric] for plot in self.plots]
@@ -346,6 +426,16 @@ class Stand(object):
         else:
             raise Exception('Invalid date')
 
+    def _add_logs_to_table_heads(self, max_logs):
+        """Adds log headers to table data depending on the maximum number of logs from trees within the stand"""
+        master = []
+        for i in range(2, max_logs + 1):
+            for name in ['Length', 'Grade', 'Defect']:
+                master.append(f'Log {i} {name}')
+            if i < max_logs:
+                master.append('Between Logs Feet')
+        return master
+
     def _get_stats(self, data):
         """Runs the statistical calculations on a set of the stand conditions data, returns an updated sub dict"""
         m = mean(data)
@@ -378,16 +468,18 @@ class Stand(object):
 """EXAMPLE WORK FLOWS"""
 
 if __name__ == '__main__':
-    from thin import (ThinTPA,
-                      ThinBA,
-                      ThinRD)
-    from fvs import FVS
+    from treetopper.thin import (ThinTPA,
+                                 ThinBA,
+                                 ThinRD)
+    from treetopper.fvs import FVS
 
     def workflow_1():
         """Workflow 1 will create a quick cruise stand from manually entered trees and plots and will then show a console report.
 
            Using the ThinTPA class, we will thin the stand to a Trees per Acre of 80 considering all species and diameter ranges
-           and then will show a console report of the thinning"""
+           and then will show a console report of the thinning
+
+           Finally we will export the stand's tree-level data (stand.table_data) to a CSV file in the current working directory"""
 
         stand = Stand('WF1', -20)
         plot_factor = stand.plot_factor
@@ -411,12 +503,16 @@ if __name__ == '__main__':
         thin80tpa = ThinTPA(stand, 80)
         thin80tpa.console_report()
 
+        stand.table_to_csv('example_csv_export.csv')
+
 
     def workflow_2():
         """Workflow 2 will create a full cruise stand from manually entered trees and plots and will then show a console report.
 
            Using the ThinBA class, we will thin the stand to a BA/ac of 120 considering only DF and WH and all diameter ranges
-           and then will show a console report of the thinning"""
+           and then will show a console report of the thinning
+
+           Finally we will export the stand's tree-level data (stand.table_data) to an Excel file in the current working directory"""
 
         stand = Stand('WF2', 33.3)
         plot_factor = stand.plot_factor
@@ -448,6 +544,8 @@ if __name__ == '__main__':
         thin120ba = ThinBA(stand, 120, species_to_cut=['DF', 'WH'])
         thin120ba.console_report()
 
+        stand.table_to_excel('example_xlsx_export.xlsx')
+
 
     def workflow_3():
         """Workflow 3 will create a quick cruise stand from importing a stand from a quick cruise Excel file and show a console report.
